<?php

    /**
    * @package mod-scheduler
    * @category mod
    * @author Gustav Delius, Valery Fremaux > 1.8
    *
    * This page prints the screen view for the student
    */

    /**
    *
    */
    if (!defined('MOODLE_INTERNAL')){
        die('Direct access to this script is forbidden!');
    }

    if ($action){
        include $CFG->dirroot.'/mod/scheduler/studentview.controller.php';
    }

    $mygroups = groups_get_all_groups($course->id, $USER->id, $cm->groupingid, 'g.id, g.name');
    $forcegroupappoint  = ( $cm->groupmode ? $scheduler->forcegroupappoint : 0 ); // ULPGC ecastro
    $currentgroup = optional_param('appointgroup', 0, PARAM_INT);
    if(!$groupmode){
        $appointgroup = 0;
    }

/// printing head information

    print_heading($scheduler->name, 'center', 4, 'h1');
    if (!empty($scheduler->description)){
        print_simple_box($scheduler->description, 'center', '80%');
    }

    // clean all late slots (for every body, anyway, they are passed !!)
    // scheduler_free_late_unused_slots($scheduler->id); //ULPGC ecastro, move to cron

/// get information about appointment attention

    $sql = '
        SELECT
           COUNT(*)
        FROM
            {scheduler_slots} s,
            {scheduler_appointment} a
        WHERE
            s.id = a.slotid AND
            a.studentid = ? AND
            a.attended = 1 AND
            s.schedulerid = ?
    ';
    $hasattended = $DB->count_records_sql($sql, array($USER->id, $scheduler->id));

/// get available slots

/* ULPGC ecastro SQl for unused slots
SELECT * FROM `mdl_scheduler_slots` s
LEFT JOIN `mdl_scheduler_appointment` a ON s.id = a.slotid
WHERE s.schedulerid = 1 AND (a.slotid IS NULL)
*/

    $latetime = time() + 3600 * $scheduler->reuseguardtime; // ULPGC ecastro to enforce delay for appointments

    $haveunattendedappointments = false;
    if ($slots = scheduler_get_available_slots($USER->id, $scheduler->id, true)) {
        $minhidedate = 0; // very far in the past
        $studentSlots = array();
        $studentAttendedSlots = array();
        foreach($slots as $slot) {
            /// check if other appointement is not "on the way". Student could not apply to it.
            if (scheduler_get_conflicts($scheduler->id, $slot->starttime, $slot->starttime + $slot->duration * 60, 0, $USER->id, SCHEDULER_OTHERS)){
                 continue;
            }

            /// check if not mine and late, don't care
            if (!$slot->appointedbyme && $slot->starttime + (60 * $slot->duration) < time()){
                continue;
            }

            /// check what to print in groupsession indication
            if ($slot->exclusivity == 0){
                $slot->groupsession = get_string('yes');
            } else {
                // $consumed = scheduler_get_consumed($scheduler->id, $slot->starttime, $slot->starttime + $slot->duration * 60, $slot->teacher);
                if ($slot->exclusivity > $slot->population){
                    $slot->groupsession = get_string('limited', 'scheduler', ($slot->exclusivity - $slot->population)."/".$slot->exclusivity);
                } else { // should not be visible to students
                    $slot->groupsession = get_string('complete', 'scheduler');
                }
            }

/// examine slot situations and elects those which have sense for the current student

            // I am in slot, unconditionnally
            if ($slot->appointedbyme) {
                if ($slot->attended){
                    $studentAttendedSlots[$slot->starttime.'_'.$slot->teacherid] = $slot;
                } else {
                    $studentSlots[$slot->starttime.'_'.$slot->teacherid] = $slot;
                }
                // binary or and and required here to calculate properly
                $haveunattendedappointments = $haveunattendedappointments | ($slot->appointedbyme & !$slot->attended);
            } else {
                // slot is free
                // ULPGC ecastro check first if too late to appoint
                if($slot->starttime < $latetime) {
                    continue;
                }
                if (!$slot->appointed) {

                    //if student is only allowed one appointment and this student has already had their then skip this record
                    if (($hasattended) and ($scheduler->schedulermode == 'oneonly')){
                        continue;
                    }
                    elseif ($slot->hideuntil <= time()){
                        $studentSlots[$slot->starttime.'_'.$slot->teacherid] = $slot;
                    }
                    $minhidedate = ($slot->hideuntil < $minhidedate || $minhidedate == 0) ? $slot->hideuntil : $minhidedate ;
                }
                // slot is booked by another student, group booking is allowed and there is still room
                elseif ($slot->appointed and (($slot->exclusivity == 0) || ($slot->exclusivity > $slot->population))) {
                    // there is already a record fot this time/teacher : sure its our's
                    if (array_key_exists($slot->starttime.'_'.$slot->teacherid, $studentSlots)) continue;
                    // else record the slot with this user (not me).
                    $studentSlots[$slot->starttime.'_'.$slot->teacherid] = $slot;
                }
            }
        }

/// prepare attended slot table

        if (count($studentAttendedSlots)){
            print_heading(get_string('attendedslots' ,'scheduler'));
            $table->head  = array ($strdate, format_string($scheduler->staffrolename), $strnote, $strgrade);
            $table->align = array ('LEFT', 'CENTER', 'LEFT', 'LEFT');
            $table->size = array ('', '', '40%', '150');
            $table->width = '90%';
            $table->data = array();
            $previousdate = '';
            $previoustime = 0;
            $previousendtime = 0;

            foreach($studentAttendedSlots as $key => $aSlot){
                /// preparing data
                $startdate = scheduler_userdate($aSlot->starttime,1);
                $starttime = scheduler_usertime($aSlot->starttime,1);
                $endtime = scheduler_usertime($aSlot->starttime + ($aSlot->duration * 60),1);
                $startdatestr = ($startdate == $previousdate) ? '' : $startdate ;
                $starttimestr = ($starttime == $previoustime) ? '' : $starttime ;
                $endtimestr = ($endtime == $previousendtime) ? '' : $endtime ;
                $studentappointment = $DB->get_record('scheduler_appointment', array('slotid' => $aSlot->id, 'studentid' => $USER->id));
                if ($scheduler->scale  > 0){
                    $studentappointment->grade = $studentappointment->grade.'/'.$scheduler->scale;
                }

                if (has_capability('mod/scheduler:seeotherstudentsresults', $context)){
                    $appointments = scheduler_get_appointments($aSlot->id);
                    $collegues = '';
                    foreach($appointments as $appstudent){
                        $grade = $appstudent->grade;
                        if ($scheduler->scale > 0){
                            $grade = $grade . '/' . $scheduler->scale;
                        }
                        $student = $DB->get_record('user', array('id' => $appstudent->studentid));
                        $picture = print_user_picture($appstudent->studentid, $course->id, $student->picture, 0, true, true);
                        $name = fullname($student);
                        if ($appstudent->studentid == $USER->id) $name = "<b>$name</b>" ; // it's me !!
                        $collegues .= " $picture $name ($grade)<br/>";
                    }
                } else {
                  $collegues = $studentappointment->grade;
                }

                $studentnotes1 = '';
                $studentnotes2 = '';
                if ($aSlot->notes != ''){
                    $studentnotes1 = '<div class="slotnotes">';
                    $studentnotes1 .= '<b>'.get_string('yourslotnotes', 'scheduler').'</b><br/>';
                    $studentnotes1 .= format_string($aSlot->notes).'</div>';
                }
                if ($studentappointment->appointmentnote != ''){
                    $studentnotes2 .= '<div class="appointmentnote">';
                    $studentnotes2 .= '<b>'.get_string('yourappointmentnote', 'scheduler').'</b><br/>';
                    $studentnotes2 .= format_string($studentappointment->appointmentnote).'</div>';
                }
                $studentnotes = "{$studentnotes1}{$studentnotes2}";

                // recording data into table
                $table->data[] = array ("<span class=\"attended\">$startdatestr</span><br/><div class=\"timelabel\">[$starttimestr - $endtimestr]</div>", "<a href=\"../../user/view.php?id={$aSlot->teacherid}&amp;course={$scheduler->course}\">".fullname($DB->get_record('user', array('id' => $aSlot->teacherid), 'u.id, firstname, lastname')).'</a>',$studentnotes, $collegues);

                $previoustime = $starttime;
                $previousendtime = $endtime;
                $previousdate = $startdate;
            }

            print_table($table);
        }





/// prepare appointable slot table
        print_heading(get_string('slots' ,'scheduler'));
        if (scheduler_has_slot($USER->id, $scheduler, true)) {
            $message = get_string('welcomebackstudent', 'scheduler');
        } else {
            $message = get_string('welcomenewstudent', 'scheduler');
        }
        print_box($message, 'generalbox boxwidthwide boxaligncenter ');

        unset($table);
        $table->head  = array ($strdate, $strstart, $strend, get_string('choice', 'scheduler'), format_string($scheduler->staffrolename), get_string('groupsession', 'scheduler'));
        $table->align = array ('LEFT', 'LEFT', 'CENTER', 'CENTER', 'LEFT');
        $table->width = "100%";
        $table->data = array();
        $previousdate = '';
        $previoustime = 0;
        $previousendtime = 0;
        $canappoint = false;
        $appointnote = '';
        foreach($studentSlots as $key => $aSlot){
            $startdate = scheduler_userdate($aSlot->starttime,1);
            $starttime = scheduler_usertime($aSlot->starttime,1);
            $endtime = scheduler_usertime($aSlot->starttime + ($aSlot->duration * 60),1);
            $startdatestr = ($startdate == $previousdate) ? '' : $startdate ;
            $starttimestr = ($starttime == $previoustime) ? '' : $starttime ;
            $endtimestr = ($endtime == $previousendtime) ? '' : $endtime ;
            if ($aSlot->appointed and has_capability('mod/scheduler:seeotherstudentsbooking', $context)){
                $appointments = scheduler_get_appointments($aSlot->id);
                $collegues = "<div style=\"visibility:hidden; display:none; text-align: left; \" id=\"collegues{$aSlot->id}\"><br/>";
                foreach($appointments as $appstudent){
                    $student = $DB->get_record('user', array('id' => $appstudent->studentid), 'u.id, firstname, lastname, email, picture, emailstop');
                    $picture = print_user_picture($appstudent->studentid, $course->id, $student->picture, 0, true, true);
                    $name = "<a href=\"../../user/view.php?id={$appstudent->studentid}&amp;course=$scheduler->course\">".fullname($student).'</a>';
                    $collegues .= " $picture $name<br/>";
                }
                $collegues .= '</div>';
                $aSlot->groupsession .= " <a href=\"javascript:toggleVisibility('{$aSlot->id}', 'collegues')\"><img name=\"group<?php p($aSlot->id) ?>\" src=\"{$CFG->pixpath}/t/switch_plus.gif\" border=\"0\" title=\"".get_string('whosthere', 'scheduler')."\"></a> {$collegues}";
            }
            $notes = '';
            if($aSlot->notes) {
                $notes = '<div style="float:right">&nbsp;<a href="wwww" >'.get_string('notes', 'scheduler').
                                        '<img name="notes" src="'.$CFG->pixpath.'/t/feedback.gif" border="0" title="'.get_string('appointmentnotes', 'scheduler').'"></a></div>';
            }
            $teacherlink = '<div style="float:left">'."<a href=\"../../user/view.php?id={$aSlot->teacherid}&amp;course=$scheduler->course\">".fullname($DB->get_record('user', array('id' => $aSlot->teacherid), 'u.id, fistname, lastname'))."</a></div>";
            if ($aSlot->appointedbyme and !$aSlot->attended){
                $radio = "<input type=\"radio\" name=\"slotid\" value=\"{$aSlot->id}\" checked=\"checked\" />\n";
                //$table->data[] = array ("<b>$startdatestr</b>", "<b>$starttime</b>", "<b>$endtime</b>", $radio, "<b>"."<a href=\"../../user/view.php?id={$aSlot->teacherid}&amp;course=$scheduler->course\">".fullname($DB->get_record('user', array('id' => $aSlot->teacherid), 'id, firstname, lastname'))."</a>$notes</b>",'<b>'.$aSlot->groupsession.'</b>');
                $table->data[] = array ("<b>$startdatestr</b>", "<b>$starttime</b>", "<b>$endtime</b>", $radio, "<b>".$teacherlink.$notes."</b>",'<b>'.$aSlot->groupsession.'</b>');
                $appointnote = format_string($DB->get_field('scheduler_appointment', 'appointmentnote', array('slotid' => $aSlot->id, 'studentid' => $USER->id)), FORMAT_HTML);
            } else {

                $canappoint = true;
                $canusegroup = ($aSlot->appointed) ? 0 : 1;
                $radio = "<input type=\"radio\" name=\"slotid\" value=\"{$aSlot->id}\" onclick=\"checkGroupAppointment($canusegroup)\" />\n";
                //$table->data[] = array ($startdatestr, $starttimestr, $endtimestr, $radio, "<a href=\"../../user/view.php?id={$aSlot->teacherid}&amp;course={$scheduler->course}\">".fullname($DB->get_record('user', array('id' => $aSlot->teacherid), 'id, firstname, lastname')).'</a>'.$notes, $aSlot->groupsession);
                $table->data[] = array ($startdatestr, $starttimestr, $endtimestr, $radio, $teacherlink.$notes, $aSlot->groupsession);
            }
            $previoustime = $starttime;
            $previousendtime = $endtime;
            $previousdate = $startdate;
        }

/// print slot table

        if (count($table->data)){
            /// add some global script
            echo "<script type=\"text/javascript\">
                        function checkGroupAppointment(enable){
                            var numgroups = '".count($mygroups)."';
                            if (!enable){
                                if (numgroups > 1){ // we have a select. we must force appointsolo
                                    document.forms['appoint'].elements['appointgroup'].options[0].selected = true;
                                }
                            }
                            document.forms['appoint'].elements['appointgroup'].disabled = !enable;
                        }

                        function toggleVisibility(id, block){
                            obj = document.getElementById(block + id);
                            if (obj.style.visibility == \"hidden\"){
                                obj.style.visibility = \"visible\"
                                obj.style.display = \"block\";
                                document.images[\"group\"+id].src='".$CFG->pixpath."/t/switch_minus.gif';
                            } else {
                                obj.style.visibility = \"hidden\";
                                obj.style.display = \"none\";
                                document.images[\"group\"+id].src='".$CFG->pixpath."/t/switch_plus.gif';
                            }
                        }
                        </script> ";

            /// Add the form
            print_container_start(true, 'generalbox boxwidthwide boxaligncenter centerpara');
            echo '<form name="appoint" action="view.php" method="get">
                        <input type="hidden" name="what" value="savechoice" />
                        <input type="hidden" name="id" value="'.$cm->id.'" />';

            print_table($table);

            if ($canappoint){
                if ($forcegroupappoint && $mygroups){
                    echo '<br />'.get_string('appointfor', 'scheduler').':&nbsp;' ;
                    $groupchoice = array();
                    if($forcegroupappoint == 1) {
                        $groupchoice[0] = get_string('appointsolo','scheduler');
                    }
                    foreach($mygroups as $group){
                        $groupchoice[$group->id] = $group->name;
                    }
                    choose_from_menu($groupchoice, 'appointgroup', $currentgroup, '');
                    helpbutton('appointagroup', get_string('appointfor', 'scheduler'), 'scheduler');
                }
                echo '<br /><br /><label for="appointnote" >'.get_string('appointmentnotes', 'scheduler').': </label>';
                echo '<textarea  name="appointnote"  rows="4" cols="60" style="vertical-align: top;">'.$appointnote,'</textarea>';
                echo '<br /><input type="submit" value="'.get_string('savechoice', 'scheduler').'" /> ';
                if ($haveunattendedappointments) {
                    echo '&nbsp;<input type="submit" value="'.get_string('updatenotes', 'scheduler').'"  onclick="'."document.forms['appoint'].elements['what'].value = 'updateappnote'; ",'" /> ';
                    if(has_capability('mod/scheduler:disengage', $context)){
                        echo '&nbsp;<input type="submit" value="'.get_string('disengage', 'scheduler').'"  onclick="'."document.forms['appoint'].elements['what'].value = 'disengage'; ",'" /> ';
                    }
                }
            }

            echo '</form>';

            print_container_end();

        }
        else {
            if ($minhidedate > time()){
                $noslots = get_string('noslotsopennow', 'scheduler') .'<br/><br/>';
                $noslots .= get_string('firstslotavailable', 'scheduler') . '<span style="color:#C00000"><b>'.userdate($minhidedate).'</b></span>';
            } else {
                $noslots = get_string('noslotsavailable', 'scheduler') .'<br/><br/>';
            }
            print_simple_box($noslots, 'center', '70%');
        }
    } else {
        notify(get_string('noslots', 'scheduler'));
    }
?>