<?PHP  // $Id: view.php,v 1.28 2009/11/06 17:53:24 diml Exp $

    /**
    * This page prints a particular instance of scheduler and handles
    * top level interactions
    *
    * @package mod-scheduler
    * @category mod
    * @author Gustav Delius, Valery Fremaux
    * @version 2.0
    *
    */

    /**
    * Requires and includes
    */
    require_once('../../config.php');
    require_once($CFG->dirroot.'/mod/scheduler/lib.php');
    require_once($CFG->dirroot.'/mod/scheduler/locallib.php');

    // common parameters
    $id         = optional_param('id', '', PARAM_INT);    // Course Module ID, or
    $a          = optional_param('a', '', PARAM_INT);     // scheduler ID
    $action     = optional_param('what', 'view', PARAM_CLEAN);
    $subaction  = optional_param('subaction', '', PARAM_CLEAN);
    $page       = optional_param('page', 'allappointments', PARAM_CLEAN);
    $offset     = optional_param('offset', '');
    $usehtmleditor = false;
    $editorfields = '';

    if ($id) {
        if (! $cm = get_coursemodule_from_id('scheduler', $id)) {
            print_error('invalidcoursemodule');
        }

        if (! $course = $DB->get_record('course', array('id' => $cm->course))) {
            print_error('coursemisconf');
        }

        if (! $scheduler = $DB->get_record('scheduler', array('id' => $cm->instance))) {
            print_error('invalidcoursemodule');
        }

    } else {
        if (! $scheduler = $DB->get_record('scheduler', array('id' => $a))) {
            print_error('invalidcoursemodule');
        }
        if (! $course = $DB->get_record('course', array('id' => $scheduler->course))) {
            print_error('coursemisconf');
        }
        if (! $cm = get_coursemodule_from_instance('scheduler', $scheduler->id, $course->id)) {
            print_error('invalidcoursemodule');
        }
    }

    require_course_login($course, true, $cm);

    // echo " [$action:$subaction] "; //$$DEBUG$$

    add_to_log($course->id, 'scheduler', "$action:$subaction", "view.php?id={$cm->id}", $scheduler->id, $cm->id);
    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    $groupmode = groupmode($course, $cm);
    $forcegroupappoint  = ( $groupmode ? $scheduler->forcegroupappoint : 0 ); // ULPGC ecastro

/// This is a pre-header selector for downloded documents generation

    if (has_capability('mod/scheduler:manage', $context) || has_capability('mod/scheduler:attend', $context)) {
        if (preg_match("/downloadexcel|downloadcsv|downloadods|dodownloadcsv/", $action)){
            include($CFG->dirroot.'/mod/scheduler/downloads.php');
        }
    }

/// Print the page header

    $strschedulers = get_string('modulenameplural', 'scheduler');
    $strscheduler  = get_string('modulename', 'scheduler');
    $buttontext = update_module_button($cm->id, $course->id, $strscheduler);
    $strtime = get_string('time');
    $strdate = get_string('date', 'scheduler');
    $strstart = get_string('start', 'scheduler');
    $strend = get_string('end', 'scheduler');
    $strlocation = get_string('location', 'scheduler');
    $strname = get_string('name');
    $strseen = get_string('seen', 'scheduler');
    $strnote = get_string('comments', 'scheduler');
    $strgrade = get_string('note', 'scheduler');
    $straction = get_string('action', 'scheduler');
    $strduration = get_string('duration', 'scheduler');
    $stremail = get_string('email');

    if (!function_exists('build_navigation')){ // 1.8 branch stable timestamp
        print_header_simple($scheduler->name, '',
                     "<a href=\"index.php?id={$course->id}\">{$strschedulers}</a> -> {$scheduler->name}",
                      '', '', true, update_module_button($cm->id, $course->id, $strscheduler),
                      navmenu($course, $cm));
    } else {
    /// Print header.
        $navigation = build_navigation('', $cm);
        print_header_simple(format_string($scheduler->name), '', $navigation, '', '', true, $buttontext, navmenu($course, $cm));
    }

/// integrate module specific stylesheets

    echo '<link rel="stylesheet" href="'.$CFG->themewww.'/standard/scheduler.css" type="text/css" />';
    echo '<link rel="stylesheet" href="'.$CFG->themewww.'/'.current_theme().'/scheduler.css" type="text/css" />';

/// route to screen

    // teacher side
    if (has_capability('mod/scheduler:manage', $context)) {
        if ($action == 'viewstatistics'){
            include $CFG->dirroot.'/mod/scheduler/viewstatistics.php';
        } elseif ($action == 'viewstudent'){
            include $CFG->dirroot.'/mod/scheduler/viewstudent.php';
        } elseif ($action == 'unscheduled'){
            include $CFG->dirroot.'/mod/scheduler/unscheduledview.php';
        } elseif ($action == 'downloads'){
            include $CFG->dirroot.'/mod/scheduler/downloads.php';
        } elseif ($action == 'datelist'){
            include $CFG->dirroot.'/mod/scheduler/datelist.php';
        } elseif ($action == 'automatic'){
            include $CFG->dirroot.'/mod/scheduler/automaticview.php';
        } else {
            include $CFG->dirroot.'/mod/scheduler/teacherview.php';
        }
    }

    // student side
    elseif (has_capability('mod/scheduler:appoint', $context)) {
        include $CFG->dirroot.'/mod/scheduler/studentview.php';
    }
    // for guests
    else {
        echo "<br/>";
        print_simple_box(get_string('guestscantdoanything', 'scheduler'), 'center', '70%');
    }

/// Finish the page

    if (empty($nohtmleditorneeded) and $usehtmleditor) {
        use_html_editor($editorfields);
    }
    print_footer($course);

?>